//
// Image.hh for pekwm
// Copyright (C) 2003-2004 Claes Nasten <pekdon{@}pekdon{.}net>
//
// This program is licensed under the GNU GPL.
// See the LICENSE file for more information.
//

#include "../config.h"

#ifndef _IMAGE_HH_
#define _IMAGE_HH_

#include "pekwm.hh"

#include <string>

extern "C" {
#include <X11/Xlib.h>
#ifdef HAVE_IMLIB2
#include <Imlib2.h>
#else // !HAVE_IMLIB2
#include <X11/xpm.h>
#endif // HAVE_IMLIB2
}

class PScreen;

class Image
{
public:
	Image(Display *dpy);
	~Image(void);

	inline Pixmap getPixmap(void) { return _pixmap; }
	inline Pixmap getMask(void) { return _mask; }

	inline uint getWidth(void) const { return _width; }
	inline uint getHeight(void) const { return _height; }
	inline ImageType getType(void) const { return _image_type; }

	inline void setImageType(ImageType it) { _image_type = it; }

	bool load(const std::string &file);
	void unload(void);

	void render(Drawable dest, int x, int y, uint width = 0, uint height = 0);
	void scale(uint width, uint height);
	Image* getScaled(uint width, uint height);

private:
	Display *_dpy;

#ifdef HAVE_IMLIB2
	Imlib_Image _im_image;
#else //! HAVE_IMLIB2
	XpmImage _xpm_image;
#endif // HAVE_IMLIB2

	Pixmap _pixmap, _mask;
	uint _width, _height;

	ImageType _image_type;
};

#endif // _IMAGE_HH_
